@@ -15,6 +15,7 @@
 #define PAGE0   ((struct zeropage *)0x00000000)
 
 char commandline[CMDLINELEN];
+extern char _end, _start, _edata;
 
 int Debug = 0;
 int interactive = 0;
@@ -161,6 +162,7 @@ int
 load_kernel(int fd, unsigned *entryp, int *wide)
 {
     struct loadable loadable;
+    void *malloc_start;
     int i;
 
     if (!prepare_loadable(fd, &loadable, wide))
@@ -176,12 +178,26 @@ load_kernel(int fd, unsigned *entryp, int *wide)
     printf("\nEntry %08x first %08x n %d\n",
 		loadable.entry, loadable.first, loadable.n);
 
+    malloc_start = malloc_aligned(0, 1);
+    malloc_start -= 5*1024*1024; /* 5 MB */
+
     for (i = 0; i < loadable.n; i++)
     {
 	loadable.segment[i].mem =  PHYS(loadable.segment[i].mem);
 	printf("Segment %d load %08x size %d mediaptr 0x%lx\n",
 		i, loadable.segment[i].mem, loadable.segment[i].length,
 		loadable.segment[i].offset);
+
+	/* check not to overwrite us */
+	if (loadable.segment[i].mem >= (unsigned long)&_end &&
+	    (loadable.segment[i].mem + loadable.segment[i].length)
+			< (unsigned long)malloc_start)
+		continue;
+
+	printf("Abort: Would overwrite palo %p-%p or data %p areas.\n",
+		&_start, &_end, malloc_start);
+	return 0;
+
     }
 
     if (!load_loadable((char *)loadable.first, fd, &loadable))
@@ -215,7 +231,7 @@ load_rd(int fd, int size)
 	rd_end = rd + size;
     }
     flush_data_cache((char *)rd, size);
-    puts("\n");
+    printf("\n");
 
     return (rd_start != 0);
 }
@@ -332,9 +348,9 @@ interact(int *ok)
 
     while (1)
     {
-	puts("Current command line:\n");
-	puts(commandline);
-	puts("\n");
+	printf("Current command line:\n");
+	printf(commandline);
+	printf("\n");
 
 	p = commandline;
 	argc = 0;
@@ -357,17 +373,20 @@ interact(int *ok)
 	{
 	    printf("%2d: %s\n", i, argv[i]);
 	}
-	puts("\n"
+	printf("\n"
 	    "<#>    edit the numbered field\n"
 	    "'b'    boot with this command line\n"
 	    "'r'    restore command line\n"
 	    "'l'    list dir\n"
 	    "'x'    reset and reboot machine\n"
+#ifdef HIDDEN_COMMAND
+	    "'d'    toggle debug mode\n"
+#endif
 	    "? ");
 	numbuf[0] = '0';
 	numbuf[1] = '\0';
 	enter_text(numbuf, sizeof numbuf - 1);
-	puts("\n");
+	printf("\n");
 
 	if (numbuf[0] == 'b')
 	{
@@ -391,18 +410,28 @@ interact(int *ok)
 	if (numbuf[0] == 'x')
 	    pdc_do_reset();
 
+	if (numbuf[0] == 'd') /* hidden option! */
+	{
+	    Debug = !Debug;
+	    printf("Debug mode is now %s\n", Debug ? "on":"off");
+	    if (Debug)
+		printf("palo loaded at %p-%p.\n", &_start, &_end);
+	    continue;
+	}
+
+
 	editfield = parse_number(numbuf, &p);
 
 	if (editfield >= MAX_ARGV)
 	{
-	    puts("Too many input fields.\n");
+	    printf("Too many input fields.\n");
 	}
 	else if (editfield < argc)
 	{
 	    strncpy(fieldbuf, argv[editfield], sizeof(fieldbuf));
 	    fieldbuf[sizeof(fieldbuf)-1] = '\0';
 	    enter_text(fieldbuf, sizeof fieldbuf - 1);
-	    puts("\n");
+	    printf("\n");
 	    argv[editfield] = fieldbuf;
 	}
 
@@ -410,10 +439,44 @@ interact(int *ok)
     }
 }
 
+/*
+ * On some server models the serial port of the GSP/Management card which
+ * mirrors the console port shows up as ttyS1 instead of ttyS0.  This happens
+ * on older Linux kernels which set up a serial port for the Diva AUX port.
+ * The Linux kernel commit bcf3f1752a62 ("parisc: Hide Diva-built-in serial aux
+ * and graphics card") avoids this by hiding the Diva AUX port:
+ * https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/commit/?id=bcf3f1752a622f1372d3252d0fea8855d89812e7
+ *
+ * Beware: pdc_model_sysmodel() may return a machine name which has trailing
+ * spaces.
+ */
+static char *
+get_default_serial_console()
+{
+	char sys_model_name[81];
+	char *ttyS1_models[] = {
+		"9000/800/rp3410",
+		"9000/800/rp3420",
+		"9000/800/rp3440",
+		NULL
+	};
+
+	if (pdc_model_sysmodel(sys_model_name) == PDC_OK) {
+	   char **check = ttyS1_models;
+	   while (*check) {
+		if (strncmp(*check, sys_model_name, strlen(*check)) == 0)
+			printf("WARNING: Found rp34x0 machine. "
+				"Older Linux kernel may need console=ttyS1.\n");
+		check++;
+	   }
+	}
+
+	return "ttyS0";
+}
+
 unsigned
 iplmain(int is_interactive, char *initialstackptr, int started_wide)
 {
-    extern char _end, _edata;
     int partitioned;
     unsigned entry;
     struct firstblock f;
@@ -433,13 +496,14 @@ iplmain(int is_interactive, char *initialstackptr, int started_wide)
     firmware_init(started_wide);
     putchar('p');	/* if you get this p and no more, string storage */
 			/* in $GLOBAL$ is wrong or %dp is wrong */
-    puts("alo ipl " PALOVERSION " ");
-    puts(bld_info);
-    puts("\n");
+    printf("alo ipl " PALOVERSION " %s\n", bld_info);
     interactive = is_interactive;
-    if (Debug) printf("iplmain(%d, started %s)\n", interactive,
-	started_wide ? "wide" : "narrow");
-    if (Debug) printf("initial-sp %p\n", initialstackptr);
+    if (Debug) {
+	printf("iplmain(%d, started %s)\n", interactive,
+		started_wide ? "wide" : "narrow");
+	printf("initial-sp %p\n", initialstackptr);
+	printf("palo loaded at %p-%p.\n", &_start, &_end);
+    }
 
  restart:
 
@@ -523,7 +587,7 @@ iplmain(int is_interactive, char *initialstackptr, int started_wide)
             if(is_mux)
 	        chk_strcat(commandline, "ttyB0", CMDLINELEN, &ok);
             else
-	        chk_strcat(commandline, "ttyS0", CMDLINELEN, &ok);
+	        chk_strcat(commandline, get_default_serial_console(), CMDLINELEN, &ok);
 
 	    if (strstr(commandline, " TERM=") == 0)
 	        chk_strcat(commandline, " TERM=vt102", CMDLINELEN, &ok);
@@ -673,7 +737,6 @@ iplmain(int is_interactive, char *initialstackptr, int started_wide)
 	    }
 	}
 
-	/* FIXME!!! This *could* overwrite us -- probably should check */
 	if (!load_kernel(kernfd, &entry, &wide))
 	{
 	    die("ERROR: failed to load kernel\n");
