@@ -8,6 +8,11 @@
  * Copyright (C) Hewlett-Packard (Paul Bame) paul_bame@hp.com
  */
 
+#define PALOMAGIC "PALO"
+#define PALOVERSION "2.00"
+
+#ifndef __ASSEMBLY__
+
 #define  __KERNEL_STRICT_NAMES /* fixes build on SUSE 10 (kernel 2.6.1[2|3]) */
 
 #if !(defined(__hpux__) || defined(__hpux))
@@ -67,8 +72,6 @@ void *pa_memcpy(void *dest, const void *src, unsigned n);
 #define __swab16(x)	bswap_16(x)
 #endif /* __swab16 */
 
-#define PALOVERSION "1.97"
-
 /* size of I/O block used in HP firmware */
 #define FW_BLOCKSIZE	2048
 
@@ -85,7 +88,6 @@ void *pa_memcpy(void *dest, const void *src, unsigned n);
  *
  * kern_sz = 0 or rd_sz = 0 means no kern or ramdisk respectively.
  */
-#define PALOMAGIC "PALO"
 #define PALOHDRVERSION 5
 
 #define PFLAG_INSTALL 0x1
@@ -156,4 +158,5 @@ extern int load_partitions(int bootdev, struct diskpartition *mptab, int maxpart
 extern void print_ptab(struct diskpartition *mptab, int maxparts);
 extern void print_ptab_pretty(struct diskpartition *mptab, int maxparts);
 
+#endif /* __ASSEMBLY__ */
 #endif /* __COMMON_H */
