@@ -192,7 +192,7 @@ write_bootloader(int media, int bootloader,
 	error(13);
 
     /* Is it really a boot loader? */
-#define BLWORD1 0xe8000002
+#define BLWORD1 0xe8000032 /* palo 2.00 and above */
     if (BLWORD1 != __be32_to_cpu(*(unsigned *)blimage))
     {
 	fprintf(stderr, "ERROR: first word of boot loader was 0x%08x,"
